#!/usr/bin/env bash

set -o errexit
set -o pipefail
set -o nounset

[[ "${DEBUG:-}" == 'true' ]] && set -x

TEAMSPEAK_URL_AMD64=https://files.teamspeak-services.com/releases/server/3.10.0/teamspeak3-server_linux_amd64-3.10.0.tar.bz2
TEAMSPEAK_URL_I386=https://files.teamspeak-services.com/releases/server/3.10.0/teamspeak3-server_linux_x86-3.10.0.tar.bz2

TEAMSPEAK_DIR="teamspeak3-server_linux_amd64"

TEAMSPEAK_CONFIG_FILENAME=ts3server.ini

_generate_password ()
{
    echo $(openssl rand -hex 16)
}

_set_default ()
{
    server_ip=''
    server_port=9987
    server_query_port=10011
    server_file_transfers_port=30033
    server_rcon_password=$(_generate_password)
}

_parse_options ()
{
    command=${1:-}
    shift

    if [[ $command == '-h' ]]; then
        command=''
    fi

    for i in "$@"
    do
        case $i in
            -h|--help)
                show_help
                exit 0
            ;;
            --ip=*|--host=*)
                server_ip="${i#*=}"
                shift
            ;;
            --port=*)
                server_port="${i#*=}"
                shift
            ;;
            --query-port=*)
                server_query_port="${i#*=}"
                shift
            ;;
            --files-port=*|--file-transfers-port=*)
                server_file_transfers_port="${i#*=}"
                shift
            ;;
            --rcon-password=*)
                server_rcon_password="${i#*=}"
                shift
            ;;
        esac
    done
}

_show_help ()
{
    echo
    echo 'Usage:	./teamspeak_run.sh COMMAND [OPTIONS]'
    echo
    echo 'Options:'
    echo '      --ip=                   Server ip'
    echo '      --port=                 Server port'
    echo '      --query-port=           Server query port'
    echo '      --files-port=           File transfers port'
    echo '      --rcon-password=        Server rcon password/Server admin password'
    echo
    echo 'Commands:'
    echo '  run         Run server'
    echo '  install     Install dependencies (Java etc.)'
    echo
    echo 'Examples:'
    echo '      ./teamspeak_run.sh install'
    echo '      ./teamspeak_run.sh run --ip=0.0.0.0 --port=9987'
    echo '      ./teamspeak_run.sh run --ip=0.0.0.0 --port=9987 --query-port=10011 --files-port=30033'
    echo
}

_show_logo ()
{
    echo
    echo '
                  ###           ####
               #######################
               #######################
              #######````````````#######
            ######   ############   ######
          #####  ####             ##  ########
      ########  ###               ####  ########
     ######## ###                 ###### ########
      ###### ####     ################### ######
       ####  ####     ################### #####
       #### #####     ###           #####  ####
       #### #####     ###           #####  ####
       ####  ####     #########     ##### #####
      ###### ####     #########     ##### ######
     ######## ###       ######      #### ########
      ########  ##                 ###  ########
       ########   ###            ###   #####
            #####    ############   ######
              #######````````````#######
                #######################
                #######################
                 #####          ####'
    echo
    echo '
                 GameAP Server Runner'
    echo
    echo '----------------------------------------------------'
    echo
}

_install ()
{
    echo "Installation teamspeak"

    if [[ $(getconf LONG_BIT) == "64" ]]; then
        teamspeak_link=${TEAMSPEAK_URL_AMD64}
    else
        teamspeak_link=${TEAMSPEAK_URL_I386}
    fi


    curl --output teamspeak.tar.bz2 ${teamspeak_link}
    tar -xf teamspeak.tar.bz2
    rm teamspeak.tar.bz2

    if [[ -f ${TEAMSPEAK_CONFIG_FILENAME} ]]; then
        mv ${TEAMSPEAK_CONFIG_FILENAME} ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
    fi
}

_set_properties ()
{
    if [[ ! -f ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME} ]]; then
        return
    fi

    if [[ -n "${server_ip:-}" ]]; then
        sed -i "s/voice_ip.*$/voice_ip=${server_ip}/" ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
        sed -i "s/query_ip.*$/query_ip=${server_ip}/" ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
        sed -i "s/filetransfer_ip.*$/filetransfer_ip=${server_ip}/" ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
    fi

    if [[ -n "${server_port:-}" ]]; then
        sed -i "s/default_voice_port.*$/default_voice_port=${server_port}/" ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
    fi

    if [[ -n "${server_query_port:-}" ]]; then
        sed -i "s/query_port.*$/query_port=${server_query_port}/" ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
    fi

    if [[ -n "${server_file_transfers_port:-}" ]]; then
        sed -i "s/filetransfer_port.*$/filetransfer_port=${server_file_transfers_port}/" ${TEAMSPEAK_DIR}/${TEAMSPEAK_CONFIG_FILENAME}
    fi
}

_run ()
{
    export TS3SERVER_LICENSE="accept"

    if [[ ! -d ${TEAMSPEAK_DIR} ]]; then
        _install
    fi

    _set_properties

    cd ${TEAMSPEAK_DIR}
    ./ts3server inifile=ts3server.ini serveradmin_password=${server_rcon_password}
}

main ()
{
    _show_logo

    case ${command:-} in
        run|start)
            _run
        ;;

        install)
            _install
        ;;

        *)
            _show_help
        ;;
    esac
}

_set_default
_parse_options "$@"
main